from django.test import TestCase
from eveuniverse.models import EveSolarSystem

from evescout.models import SignaturePinger, SignatureSystem
from evescout.tests.testdata.load_eveuniverse import load_eveuniverse


class TestPinger(TestCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_eveuniverse()

    def test_create(self):
        SignaturePinger.create(1273234292821987391)
        self.assertTrue(True)

    def test_pingers_in_range(self):
        hasmijaala = EveSolarSystem.objects.get(id=30002806)
        hasmijaala_sig = SignatureSystem.create(
            00, hasmijaala, SignatureSystem.SignatureOrigin.THERA
        )

        sujarento = EveSolarSystem.objects.get(name="Sujarento")
        pinger_suj = SignaturePinger.create(000, sujarento)
        pinger_suj.min_ping_distance_jump = 1  # won't ping for hasmijaala
        pinger_suj.save()

        nagamanen = EveSolarSystem.objects.get(name="Nagamanen")
        pinger_nag = SignaturePinger.create(000, nagamanen)
        pinger_nag.min_ping_distance_jump = 2  # will ping for hasmijaala
        pinger_nag.save()

        tannolen = EveSolarSystem.objects.get(name="Tannolen")
        pinger_tan = SignaturePinger.create(000, tannolen)
        pinger_tan.always_ping = True
        pinger_tan.save()

        pingers_in_range = hasmijaala_sig.pingers_in_range()

        self.assertIn(pinger_nag, pingers_in_range)
        self.assertNotIn(pinger_suj, pingers_in_range)
        self.assertIn(pinger_tan, pingers_in_range)

    def test_uninitialized_pinger_in_range(self):
        """
        Unitialized pingers (with system still at None) were causing a crash when calling `pingers_in_range()`
        """
        oto = EveSolarSystem.objects.get(name="Oto")

        SignaturePinger.create(1)

        signature = SignatureSystem.create(
            1, oto, SignatureSystem.SignatureOrigin.THERA
        )

        pingers = signature.pingers_in_range()

        self.assertEqual(0, len(pingers))

    def test_in_range(self):
        nagamanen = EveSolarSystem.objects.get(id=30002807)
        pinger = SignaturePinger.create(00, nagamanen)
        pinger.min_ping_distance_ly = 10
        pinger.min_ping_distance_ly = 10
        pinger.save()

        hasmijaala = EveSolarSystem.objects.get(id=30002806)
        sig = SignatureSystem.create(
            00, hasmijaala, SignatureSystem.SignatureOrigin.THERA
        )

        self.assertTrue(sig.is_pinger_in_range(pinger))

    def test_name(self):
        pinger = SignaturePinger.create(1)

        self.assertEqual("Unlinked pinger 1", str(pinger))

    def test_wh_never_in_range(self):
        tama = EveSolarSystem.objects.get(name="Tama")
        pinger = SignaturePinger.create(1)
        pinger.always_ping = True
        pinger.system = tama
        pinger.save()

        wh_system = EveSolarSystem.objects.get(id=31001805)
        signature = SignatureSystem.create(
            00, wh_system, SignatureSystem.SignatureOrigin.THERA
        )

        pingers_in_range = signature.pingers_in_range()

        self.assertEqual(0, len(pingers_in_range))



"""Tasks."""

from celery import shared_task

from eveuniverse.models import EveSolarSystem

from allianceauth.services.hooks import get_extension_logger

from evescout.evescout import NoUpdatesSinceLastRequest, list_public_signatures
from evescout.models import SignatureSystem, Singleton

logger = get_extension_logger(__name__)


@shared_task
def update_all_signatures(*, ignore_last_check: bool = False):
    """
    Update all SignatureSystems from the results of a call to eve-scout

    ignore_last_check param set to true will ignore the update flag of the request.
    Data will be updated even if the auth has the most up-to-date version
    """
    logger.info("Updating all signatures from eve-scout task started")

    if ignore_last_check:
        last_check = None
        logger.debug("Ignoring last_check setting")
    else:
        last_check = Singleton.get_last_check_time()
        logger.debug("Last check setting is at %s", last_check)

    try:
        public_signatures = list_public_signatures(last_check)
    except NoUpdatesSinceLastRequest:
        logger.info("No new update on the EVE-scout API. Terminating")
        return

    existing_ids = {signature["id"] for signature in public_signatures}
    SignatureSystem.delete_disappeared_signatures(existing_ids)

    signatures_in_database = SignatureSystem.get_signature_ids_set()

    for signature_info in public_signatures:
        if int(signature_info["id"]) not in signatures_in_database:
            create_new_signature.delay(signature_info)

    Singleton.update_check_time()


@shared_task
def create_new_signature(signature_info: dict):
    """
    Creates a new signature from the information eve scout gives
    """

    logger.debug("Creating a signature from %s", signature_info)

    match signature_info["out_system_name"]:
        case "Thera":
            signature_origin = SignatureSystem.SignatureOrigin.THERA
        case "Turnur":
            signature_origin = SignatureSystem.SignatureOrigin.TURNUR
        case _:
            raise ValueError(
                f"Unexpected out_system_name value received {signature_info['out_system_name']}"
            )

    match signature_info["max_ship_size"]:
        case "small":
            wormhole_size = SignatureSystem.WormholeSize.S
        case "medium":
            wormhole_size = SignatureSystem.WormholeSize.M
        case "large":
            wormhole_size = SignatureSystem.WormholeSize.L
        case "xlarge":
            wormhole_size = SignatureSystem.WormholeSize.XL
        case "capital":
            wormhole_size = SignatureSystem.WormholeSize.C
        case "unknown" | _:
            wormhole_size = None

    solar_system, _ = EveSolarSystem.objects.get_or_create_esi(
        id=signature_info["in_system_id"]
    )

    signature_system = SignatureSystem.create(
        signature_info["id"], solar_system, signature_origin, wormhole_size
    )

    for pinger in signature_system.pingers_in_range():
        logger.info(
            "Sending ping for sig id %s to pinger id %s", signature_system.id, pinger.id
        )
        pinger.ping_new_sig(signature_system)



from discord import AutocompleteContext

from eveuniverse.models import EveSolarSystem

from evescout.cogs.utils.region_obs import REGION_OBS
from evescout.models import SignatureSystem


def search_solar_system(ctx: AutocompleteContext):
    """Return a list of potential solar systems based on the characters entered"""
    return [
        a
        for a in EveSolarSystem.objects.filter(name__icontains=ctx.value).values_list(
            "name", flat=True
        )[:10]
    ]


def possible_origins(ctx: AutocompleteContext):
    """Return a list of possible signature origins based on the characters entered"""
    return [
        a.name
        for a in SignatureSystem.SignatureOrigin
        if a.name.startswith(ctx.value.upper())
    ]


def possible_regions(ctx: AutocompleteContext):
    """Return a list of possible regions based on the characters §entered"""
    return [
        region_name
        for region_name in REGION_OBS.keys()
        if region_name.upper().startswith(ctx.value.upper())
    ]



"""Global utility functions for the module"""

from typing import Optional

from discord import Embed

from esi.clients import EsiClientProvider
from eveuniverse.models import EveSolarSystem

from allianceauth.services.hooks import get_extension_logger

from . import __version__

logger = get_extension_logger(__name__)

esi = EsiClientProvider(app_info_text=f"aa-evescout v{__version__}")


def default_embed(title: str) -> Embed:
    """Returns the default embed layout to use accross the app"""
    e = Embed(
        title=title,
    )
    e.set_author(
        name="EVE-scout",
        url="https://eve-scout.com/#/",
        icon_url="https://images.evetech.net/alliances/99005130/logo?size=128",
    )

    return e


# pylint: disable=inconsistent-return-statements
def find_solar_system_from_str(solar_system_name: str) -> Optional[EveSolarSystem]:
    """
    Will attempt to load a system from it's string
    """
    logger.debug("Trying to find solar system for name %s", solar_system_name)
    try:
        solar_system = EveSolarSystem.objects.get(name=solar_system_name)
        return solar_system
    except EveSolarSystem.DoesNotExist:
        logger.debug(
            "Couldn't find %s in database, trying to find the id from esi",
            solar_system_name,
        )
        solar_system_id_result = esi.client.Universe.post_universe_ids(
            names=[solar_system_name]
        ).result()
        if id_list := solar_system_id_result["systems"]:
            solar_system_id = id_list[0]["id"]
            solar_system, _ = EveSolarSystem.objects.get_or_create_esi(
                id=solar_system_id
            )
            logger.info(
                "Created system id %s matching name %s",
                solar_system.id,
                solar_system_name,
            )
            return solar_system

    logger.error("Couldn't find a matching id for %s", solar_system_name)
    return



from django.test import TestCase
from eveuniverse.tools.testdata import ModelSpec, create_testdata

from . import test_data_filename


class CreateEveUniverseTestData(TestCase):
    def test_create_testdata(self):
        test_data_spec = [
            ModelSpec(
                "EveSolarSystem",
                ids=[
                    # Isoma constellation
                    30002806,
                    30002807,
                    30002808,
                    30002809,
                    30002810,
                    30002811,
                    30002812,
                    30002813,
                    # Random wh
                    31001805,
                ],
            ),
        ]
        create_testdata(test_data_spec, test_data_filename())



"""
All API calls communicating with https://eve-scout.com/
"""

from datetime import datetime
from enum import Enum
from typing import Optional

import requests

from eveuniverse.models import EveSolarSystem

from allianceauth.services.hooks import get_extension_logger

from . import __version__, repo_url

BASE_URL = "https://api.eve-scout.com"

logger = get_extension_logger(__name__)


class ScoutedSystems(Enum):
    """
    Enumeration of systems the API is tracking
    In the current state it's only Thera and Turnur
    """

    THERA = "thera"
    TURNUR = "turnur"


class RoutingPreference(Enum):
    """Different ways to route for the pathfinding"""

    SAFER = "safer"
    SHORTEST = "shortest"
    SHORTEST_GATES = "shortest-gates"


class NoUpdatesSinceLastRequest(Exception):
    """Signals that since the last time the API was called data hasn't changed"""


def call_eve_scout_api(
    url_path: str, query_parameters: Optional[dict[str, str]] = None
):
    """Makes the call to the eve-scout API"""
    logger.debug("Querying %s path with %s params", url_path, query_parameters)
    user_agent = f"aa-evescout v{__version__} {repo_url}"
    r = requests.get(
        f"{BASE_URL}/{url_path}",
        params=query_parameters,
        headers={"User-Agent": user_agent},
        timeout=10,
    )
    logger.debug("%s returned a code %s", url_path, r.status_code)
    r.raise_for_status()

    return r.json(), r.headers


def list_public_signatures(last_check: Optional[datetime] = None) -> list[dict]:
    """
    Calls the /v2/public/signatures
    Doc: https://api.eve-scout.com/ui/#get-/v2/public/signatures

    Note: here a check of the endpoint headers will be performed.
    An error returned if there has been no modifications since `last_check` was called.
    """
    logger.info("Requesting all sigs from the API")

    response = call_eve_scout_api("v2/public/signatures")

    if last_check and response[1].get("x-last-signaleer-hub-interaction"):
        # last_update = datetime.fromisoformat(
        #     response[1]["X-Last-Signaleer-Hub-Interaction"][:-5] + "Z"
        # )
        last_update = datetime.strptime(
            response[1]["x-last-signaleer-hub-interaction"], "%Y-%m-%dT%H:%M:%S.%f%z"
        )
        if last_check > last_update:
            logger.info("No updates in the backend since our last successful update")
            raise NoUpdatesSinceLastRequest

    logger.debug("Successfully retrieved data from EVE-scout %s", response)
    return response[0]


def routes_signatures(
    from_system: EveSolarSystem,
    system_name: ScoutedSystems,
    preference: RoutingPreference = RoutingPreference.SAFER,
) -> list[dict]:
    """
    Calls the /v2/public/routes/signatures endpoint
    Doc: https://api.eve-scout.com/ui/#get-/v2/public/routes/signatures
    """
    logger.info(
        "Requesting eve-scout routes from %s to %s with pref %s",
        from_system,
        system_name,
        preference,
    )

    return call_eve_scout_api(
        "v2/public/routes/signatures",
        {
            "from": from_system.name,
            "system_name": system_name.value,
            "preference": preference.value,
        },
    )[0]


def routes_jove_observatories(
    from_system: EveSolarSystem, preference: RoutingPreference = RoutingPreference.SAFER
) -> list[dict]:
    """
    Calls the /v2/public/routes/joveobservatories endpoint
    Doc: https://api.eve-scout.com/ui/#get-/v2/public/routes/joveobservatories
    """
    logger.info(
        "Requesting jove observatories around %s with preference %s",
        from_system,
        preference,
    )

    return call_eve_scout_api(
        "v2/public/routes/joveobservatories",
        {
            "from": from_system.name,
            "preference": preference.value,
        },
    )[0]



"""Module listing all jove observatories in EVE regions"""

REGION_OBS = {
    "Aridia": "https://evemaps.dotlan.net/map/Aridia/Bazadod,Chibi,Defsunun,Esubara,Fihrneh,Hoseen,Huna,Isid,Nema,Nouta,Pserz,Soza#npc-delta",
    "Branch": "http://evemaps.dotlan.net/map/Branch/KMC-WI:AH-B84:O-JPKH:CX-1XF:Q-NJZ4:Z-K495:52G-NZ:YG-82V:T-Q2DD:LRWD-B:M-HU4V:P7Z-R3:C-LP3N:J52-BH:SVB-RE:C-VGYO:NEH-CS:4DTQ-K:W-4FA9:2B7A-3:BU-IU4:I-7JR4#npc_delta",
    "Cache": "http://evemaps.dotlan.net/map/Cache/0LY-W1:I6-SYN:LMM7-L:M-CNUD:U-QMOA:JZ-B5Y:U93O-A:NG-M8K#npc_delta",
    "Catch": "http://evemaps.dotlan.net/map/Catch/25S-6P:3GD6-8:4-07MU:7LHB-Z:9-8GBA:AX-DOT:B-XJX4:E3-SDZ:EX-0LQ:F4R2-Q:F9E-KX:G-AOTH:GE-94X:I-8D0G:IS-R7P:JA-O6J:JWZ2-V:L7XS-5:OGL8-Q:SNFV-I:ZQ-Z3Y#npc_delta",
    "Cloud Ring": "http://evemaps.dotlan.net/map/Cloud_Ring/XT-R36:77-KDQ:O-IVNH:OU-X3P:6-4V20:8R-RTB:G8AD-C:MJYW-3:QA1-BT:28O-JY:X-M9ON#npc_delta",
    "Cobalt Edge": "http://evemaps.dotlan.net/map/Cobalt_Edge/FV-YEA:5E-EZC:Y-RAW3:X-41DA:E-BYOS:5HN-D6:E-B957:87-1PM:5ED-4E:FV1-RQ:L-Z9NB:DK0-N8:CHP-76:1GT-MA:HB-5L3#npc_delta",
    "Curse": "http://evemaps.dotlan.net/map/Curse/ES-UWY:G-G78S:J7A-UR:K-MGJ7:V7D-JD:Y-K50G#npc_delta",
    "Deklein": "http://evemaps.dotlan.net/map/Deklein/TXME-A:S-DN5M:MXX5-9:N-TFXK:33RB-O:E-FIC0:ZOYW-O:5W3-DG:3JN9-Q:MZ1E-P:XCBK-X:U-TJ7Y#npc_delta",
    "Delve": "http://evemaps.dotlan.net/map/Delve/Z3V-1W:HM-XR2:YZ9-F6:IP6V-X:D-W7F0:QY6-RK:ZXB-VC:8F-TK3:F-9PXR:KBAK-I:M-SRKS#npc_delta",
    "Detorid": "http://evemaps.dotlan.net/map/Detorid/0-G8NO:1KAW-T:3-LJW3:4NDT-W:5J4K-9:62O-UE:7-A6XV:9-02G0:9N-0HF:BOZ1-O:D-0UI0:E51-JE:PQRE-W:P7MI-T:GR-X26:L8-WNE:GC-LTF:NB-ALM:L-QQ6P:XA5-TY:RZ-PIY:TYB-69:T-0JWP:U-MFTL:S-51XG:Z-2Y2Y:SY-UWN:I-9GI1#npc_delta",
    "Esoteria": "http://evemaps.dotlan.net/map/Esoteria/G2-INZ:JAUD-V:D-FVI7:CR-0E5:F-UVBV:G-YZUX:6EK-BV:NIZJ-0:Q1-R7K:R-ARKN:SN9S-N:PE-H02:G-4H4C:HHE5-L:P9F-ZG:C-PEWN:IPX-H5:YRV-MZ#npc_delta",
    "Etherium Reach": "http://evemaps.dotlan.net/map/Etherium_Reach/9P-870:KGT3-6:KMH-J1:RO-0PZ:IRD-HU:4-QDIX:89-JPE:8KE-YS:3IK-7O:1PF-BC:WPV-JN:CT8K-0:F69O-M#npc_delta",
    "Fade": "http://evemaps.dotlan.net/map/Fade/O1Y-ED:X36Y-G:0-ARFO#npc_delta",
    "Feythabolis": "http://evemaps.dotlan.net/map/Feythabolis/UD-VZW:OXC-UL:X6-J6R:DUU1-K:K-X5AX:Z-PNIA:PO-3QW:U-BXU9:6O-XIO:2UK4-N:QK-CDG:2-F3OE:CL-J9W:UB5Z-3:3-BADZ:P8-BKO:I9-ZQZ:ZID-LE#npc_delta",
    "Fountain": "http://evemaps.dotlan.net/map/Fountain/AC2E-3:R3W-XU:SPLE-Y:Q-XEB3:JGOW-Y:KCT-0A:IGE-RI:Z9PP-H:7X-02R:6F-H3W:L-1SW8:CHA2-Q:UAYL-F:A-HZYL:H-S80W:15U-JY:HMF-9D:7BX-6F:P5-EFH:YRNJ-8:G-UTHL:I-CUVX:87XQ-0:V6-NY1:PXF-RF:Z-YN5Y#npc_delta",
    "Geminate": "http://evemaps.dotlan.net/map/Geminate/SR-KBB:FDZ4-A:WH-2EZ:QKTR-L:BWF-ZZ:4-CUM5:39-DGG:K42-IE:LX-ZOJ:6L78-1:L-TOFR:TJM-JJ:D-I9HJ:HKYW-T:4K0N-J:B-F1MI:O2O-2X:TDE4-H#npc_delta",
    "Great Wildlands": "http://evemaps.dotlan.net/map/Great_Wildlands/4M-P1I:7Q-8Z2:8DL-CP:B-VIP9:DE71-9:F5-CGW:HB-1NJ:IX8-JB:N-RAEL:NE-3GR:YUY-LM:Y4-GQV:UT-UZB:TET3-B:ZM-DNR:QQ3-YI:QM-20X:R-ESG0:UNJ-GX:V89M-R#npc_delta",
    "Immensea": "http://evemaps.dotlan.net/map/Immensea/NS2L-4:Y19P-1:O7-7UX:CJNF-J:AC-7LZ:Y-FZ5N:U9U-TQ:6-I162:Y-C4AL:FRTC-5:R-ORB7#npc_delta",
    "Impass": "http://evemaps.dotlan.net/map/Impass/4OIV-X:DY-40Z:E7VE-V:01TG-J:PZMA-E:Z-N9IP:6E-MOW:X-0CKQ:GU-9F4:DDI-B7#npc_delta",
    "Insmother": "http://evemaps.dotlan.net/map/Insmother/0-VG7A:1ZF-PJ:4CJ-AC:5-2PQU:5C-RPA:88A-RA:B-II34:C1G-XC:EKPB-3:EUU-4N:F3-8X2:F39H-1:GDHN-K:GK5Z-T:HL-VZX:KD-KPR:L5-UWT:O-7LAI:XQP-9C:V-QXXK:MOCW-2:N-O53U:N7-BIY:ZO-4AR:RD-FWY:VBPT-T#npc_delta",
    "Malpais": "http://evemaps.dotlan.net/map/Malpais/1NZV-7:7T-0QS:APES-G:DAI-SH:F-TQWO:GL6S-2:H-EBQG:HG-YEQ:IBOX-2:JA-G0T:N-YLOE:NBW-GD:NSI-MW:POQP-K:PWPY-4:QSCO-D:RUF3-O:S5W-1Z:VVB-QH:Y-XZA7:XTVZ-E:ZOPZ-6#npc_delta",
    "Oasa": "http://evemaps.dotlan.net/map/Oasa/MZLW-9:DGDT-3:IG-4OF:Y-770C:X-Z4JW:6U-MFQ:IO-R2S:U9SE-N:3-JG3X:KED-2O:U-RELP:HO4E-Q:X-CYNC:RJBC-I:XKM-DE:JXQJ-B#npc_delta",
    "Omist": "http://evemaps.dotlan.net/map/Omist/9MWZ-B:C-WPWH:KR8-27:M9-FIB:DJK-67:X-EHHD:L-AS00:99-0GS:H90-C9:Z-7OK1:OKEO-X:7-8EOE#npc_delta",
    "Outer Passage": "http://evemaps.dotlan.net/map/Outer_Passage/QHH-13:XUPK-Z:WIO-OL:YQM-P1:TFPT-U:QFRV-2:8-AA98:2ULC-J:0-4VQL:QOK-SX:KGCF-5:2WU-XT:J7X-VN:ZZK-VF:4AZV-W:MC4C-H:CNHV-M:N-I024:4O-ZRI#npc_delta",
    "Outer Ring": "http://evemaps.dotlan.net/map/Outer_Ring/C0T-77:F5FO-U:0-WVQS:JKJ-VJ:T-Z6J2:CT7-5V:A-5M31:0D-CHA#npc_delta",
    "Paragon Soul": "http://evemaps.dotlan.net/map/Paragon_Soul/ARBX-9:O-MCZR:H8-ZTO:3PPT-9:GQ2S-8:LD-2VL:O-N589#npc_delta",
    "Period Basis": "http://evemaps.dotlan.net/map/Period_Basis/TCAG-3:VQE-CN:E-DOF2:08S-39:RYQC-I:Z-M5A1#npc_delta",
    "Perrigen Falls": "http://evemaps.dotlan.net/map/Perrigen_Falls/XU7-CH:E-WMT7:LW-YEW:OP7-BP:PE-SAM:RY-2FX:H-29TM:BEG-RL:MJ-5F9:JZ-UQC:JPEZ-R:TAL1-3:4-1ECP:M4-KX5:J9A-BH:A1F-22:9-ZA4Z:O-QKSM:VWES-Y:SY-OLX:PFV-ZH:MTO2-2:0-U2M4#npc_delta",
    "Providence": "http://evemaps.dotlan.net/map/Providence/9UY4-H:FSW-3C:MH9C-S:QBL-BV:QR-K85:OXIY-V:D-6WS1:TU-O0T:GN7-XY:8P9-BM:N-RMSH:K1Y-5H:2V-CS5:HP-6Z6:I-MGAB#npc_delta",
    "Pure Blind": "http://evemaps.dotlan.net/map/Pure_Blind/KQK1-2:JC-YX8:12YA-2:U-INPD:WW-KGD:R-2R0G:5ZXX-K:4-ABS8:S-MDYI:RZC-16:UC3H-Y:RORZ-H:UI-8ZE#npc_delta",
    "Querious": "http://evemaps.dotlan.net/map/Querious/A3-LOG:O3L-95:TV8-HS:MKD-O8:A-BO4V:8B-SAJ:UYU-VV:W6V-VM:OGY-6D:UVHO-F:Z-UZZN:RF-CN3:KEJY-U:ED-L9T:E-VKJV:3D5K-R#npc_delta",
    "Scalding Pass": "http://evemaps.dotlan.net/map/Scalding_Pass/1QZ-Y9:2-KPW6:4E-EZS:6-KPAB:9PX2-F:F2-NXA:FYD-TO:GN-PDU:H5N-V7:IAK-JW:JLO-Z3:K7-LDX:X9V-15:Q-K2T7:LBC-AW:NSBE-L:WV0D-1:U2-28D:LQ-OAI:Q-NA5H:WFFE-4:V-F6DQ#npc_delta",
    "Stain": "http://evemaps.dotlan.net/map/Stain/LGK-VP:WHG2-7:Y1-UQ2:G-ME2K:FV-SE8:2IGP-1:37S-KO:A-GPTM:WEQT-K:42-UOW:UKYS-5:X5O1-L:TL-T9Z:B-G1LG:J-AYLV:CJF-1P:L6B-0N:6QBH-S:JU-UYK:4XW2-D:373Z-7:2B-3M4:LGL-SD:O-CT8N:XFBE-T#npc_delta",
    "Syndicate": "http://evemaps.dotlan.net/map/Syndicate/Poitot:FD-MLJ:6-CZ49:ZN0-SR:T-LIWS:5-T0PZ:617I-I:5-FGQI:BY-S36:0LTQ-C:9U-TTJ:KFR-ZE:5-VKCN:57-YRU:51-5XG:DUV-5Y:CIS-7X:EU0I-T:2G38-I#npc_delta",
    "Tenal": "http://evemaps.dotlan.net/map/Tenal/3DR-CR:EOY-BG:PNS7-J:KMV-CQ:S-1LIO:5-O8B1:WV-0R2:SZ6-TA:6OYQ-Z:A1RR-M:ZK-YQ3:Q1U-IU:7D-PAT:T-K10W#npc_delta",
    "Tenerifis": "http://evemaps.dotlan.net/map/Tenerifis/WB-AYY:BW-WJ2:3L3N-X:Y-ORBJ:H-HWQR:ZD1-Z2:FE-6YQ:PEK-8Z:KW-OAM:JI1-SY:16AM-3:Y-EQ0C:MS1-KJ:NZW-ZO:F-ZBO0:0P-U0Q:T-AKQZ:9-980U:ZO-P5K#npc_delta",
    "The Kalevala Expanse": "http://evemaps.dotlan.net/map/The_Kalevala_Expanse/JT2I-7:BVRQ-O:SH6X-F:TA9T-P:BM-VYZ:EPCD-D:LE-67X:6FS-CZ:HPV-RJ:C3J0-O:HD-HOZ:K76A-3:R1O-GN:86L-9F:J-OAH2#npc_delta",
    "The Spire": "http://evemaps.dotlan.net/map/The_Spire/HIK-MC:B9EA-G:E-BFLT:C6C-K9:ETO-OT:KPI-OW:K-XJJT:RXA-W1:P65-TA:K-YL9T:GTB-O4:H4X-0I:OTJ9-E:M-NP5O#npc_delta",
    "Tribute": "http://evemaps.dotlan.net/map/Tribute/C2X-M5:C8VC-S:F-749O:NJ4X-S:V0DF-2:SH1-6P:WH-JCA:S8-NSQ:XD-TOV:W-UQA5#npc_delta",
    "Vale of the Silent": "http://evemaps.dotlan.net/map/Vale_of_the_Silent/0MV-4W:1N-FJ8:1VK-6B:1W-0KS:5T-KM3:6WW-28:7-K5EL:FS-RFL:Q-R3GP:P3EN-E:S-NJBB:Z-8Q65:H-NOU5:XSQ-TF:MQ-O27:BR-6XP:VI2K-J:LS9B-9:ZA0L-U:O-LR1H:MGAM-4:JZV-F4#npc_delta",
    "Venal": "http://evemaps.dotlan.net/map/Venal/N-Q5PW:Y6-HPG:9-266Q:K3JR-J:N5Y-4N:92K-H2:AJCJ-1:HBD-CC:0-XIDJ:4RX-EE:9IPC-E:EK2-ET:2PLH-3:M2-2V1:OJ-CT4#npc_delta",
    "Wicked Creek": "http://evemaps.dotlan.net/map/Wicked_Creek/3Q-VZA:4-EFLU:5NQI-E:8-OZU1:EIH-IU:G9L-LP:GPD5-0:HD-AJ7:J7-BDX:JQU-KY:HPBE-D:U104-3:R0-DMM:J-RXYN:JEQG-7#npc_delta",
}



"""Load a missing system"""

from django.core.management import BaseCommand
from eveuniverse.models import EveSolarSystem

from allianceauth.services.hooks import get_extension_logger

logger = get_extension_logger(__name__)


class Command(BaseCommand):
    help = "Loads a system missing from eveuniverse"

    def add_arguments(self, parser):
        parser.add_argument("solar_system_id", type=int)

    def handle(self, *args, **options):
        solar_system_id = options["solar_system_id"]
        logger.info("Loading system %s", solar_system_id)

        EveSolarSystem.objects.update_or_create_esi(id=solar_system_id)



"""Admin site."""

from solo.admin import SingletonModelAdmin

from django.contrib import admin

from evescout.models import SignaturePinger, SignatureSystem, Singleton

admin.site.register(Singleton, SingletonModelAdmin)


@admin.register(SignaturePinger)
class SignaturePingerAdmin(admin.ModelAdmin):
    list_display = [
        "system",
        "discord_channel_id",
        "ping_for_thera",
        "ping_for_turnur",
        "ping_type",
        "min_ping_distance_ly",
        "min_ping_distance_jump",
    ]
    fields = [
        ("system"),
        ("discord_channel_id"),
        ("always_ping"),
        ("ping_for_thera", "ping_for_turnur"),
        ("ping_type"),
        ("min_ping_distance_ly", "min_ping_distance_jump"),
    ]


@admin.register(SignatureSystem)
class SignatureSystemAdmin(admin.ModelAdmin):
    list_display = ["system", "origin", "size"]
    list_filter = ["origin", "size"]

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False



"""Example plugin app for Alliance Auth."""

# pylint: disable = invalid-name
default_app_config = "evescout.apps.ExampleConfig"

__version__ = "1.1.1"

repo_url = "https://gitlab.com/r0kym/aa-evescout"



from django.core.management import BaseCommand

from allianceauth.services.hooks import get_extension_logger

from evescout.tasks import update_all_signatures

logger = get_extension_logger(__name__)


class Command(BaseCommand):
    help = "Updates all signatures from the eve-scout API"

    def handle(self, *args, **options):
        logger.info("Updating signatures from command line")
        update_all_signatures.delay(ignore_last_check=True)



from django.test import TestCase
from eveuniverse.models import EveSolarSystem

from evescout.models import SignatureSystem
from evescout.tests.testdata.load_eveuniverse import load_eveuniverse


class TestSignatures(TestCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_eveuniverse()

    def test_get_ids_set(self):
        oto = EveSolarSystem.objects.get(id=30002808)

        SignatureSystem.objects.create(
            id=1, system=oto, origin=SignatureSystem.SignatureOrigin.THERA
        )
        SignatureSystem.objects.create(
            id=2, system=oto, origin=SignatureSystem.SignatureOrigin.THERA
        )

        ids_set = SignatureSystem.get_signature_ids_set()

        self.assertEqual({1, 2}, ids_set)

    def test_disappeared_signatures(self):
        oto = EveSolarSystem.objects.get(id=30002808)

        sig_system_1 = SignatureSystem.objects.create(
            id=1, system=oto, origin=SignatureSystem.SignatureOrigin.THERA
        )
        sig_system_2 = SignatureSystem.objects.create(
            id=2, system=oto, origin=SignatureSystem.SignatureOrigin.THERA
        )

        SignatureSystem.delete_disappeared_signatures([1])

        self.assertEqual(1, SignatureSystem.objects.count())
        self.assertIn(sig_system_1, SignatureSystem.objects.all())
        self.assertNotIn(sig_system_2, SignatureSystem.objects.all())

    def test_get_closest_signatures(self):
        oto = EveSolarSystem.objects.get(name="Oto")
        sujarento = EveSolarSystem.objects.get(name="Sujarento")
        hasmijaala = EveSolarSystem.objects.get(name="Hasmijaala")
        nagamanen = EveSolarSystem.objects.get(name="Nagamanen")

        sig_system_1 = SignatureSystem.objects.create(
            id=1,
            system=sujarento,
            origin=SignatureSystem.SignatureOrigin.THERA,
        )
        sig_system_2 = SignatureSystem.objects.create(
            id=2,
            system=hasmijaala,
            origin=SignatureSystem.SignatureOrigin.THERA,
        )

        thera_output = SignatureSystem.get_closest_signature(
            oto, SignatureSystem.SignatureOrigin.THERA
        )
        self.assertEqual(2, len(thera_output))
        self.assertIn(sig_system_1, thera_output)
        self.assertIn(sig_system_2, thera_output)

        empty_output = SignatureSystem.get_closest_signature(
            oto, SignatureSystem.SignatureOrigin.TURNUR
        )
        self.assertEqual(0, len(empty_output))

        sig_system_3 = SignatureSystem.objects.create(
            id=3, system=nagamanen, origin=SignatureSystem.SignatureOrigin.TURNUR
        )

        turnur_ouput = SignatureSystem.get_closest_signature(
            oto, SignatureSystem.SignatureOrigin.TURNUR
        )
        self.assertEqual(1, len(turnur_ouput))
        self.assertIn(sig_system_3, turnur_ouput)

        thera_output = SignatureSystem.get_closest_signature(
            oto, SignatureSystem.SignatureOrigin.THERA
        )
        self.assertEqual(2, len(thera_output))
        self.assertIn(sig_system_1, thera_output)
        self.assertIn(sig_system_2, thera_output)
        self.assertNotIn(sig_system_3, thera_output)



# Generated by Django 4.2.16 on 2024-11-20 12:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("evescout", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="signaturepinger",
            name="ping_type",
            field=models.CharField(
                choices=[(" ", "None"), ("@here", "@here"), ("@everyone", "@everyone")],
                default=" ",
                help_text="Type of notification that should be sent out with the ping",
                max_length=9,
            ),
        ),
    ]



from aadiscordbot.app_settings import get_site_url
from aadiscordbot.cogs.utils.decorators import sender_has_perm
from discord import ApplicationContext, SlashCommandGroup, option
from discord.ext import commands

from django.conf import settings
from eveuniverse.models import EveSolarSystem

from allianceauth.services.hooks import get_extension_logger

from evescout.cogs.utils.autocompletes import possible_regions
from evescout.cogs.utils.region_obs import REGION_OBS
from evescout.models import SignaturePinger, SignatureSystem
from evescout.utils import default_embed, find_solar_system_from_str

from ..evescout import routes_jove_observatories
from .utils.autocompletes import possible_origins, search_solar_system

logger = get_extension_logger(__name__)


def _signature_info(signature: SignatureSystem, solar_system: EveSolarSystem) -> str:
    """Return the signature info to be put in an embed field value"""
    gates, light_years = signature.distance_from_solar_system(solar_system)
    return (
        f"Distance:\n"
        f"- {gates} gates\n"
        f"- {round(light_years, 2)} ly.\n"
        f"Size: {signature.get_size_str()}\n"
    )


class EveScout(commands.Cog):

    def __init__(self, bot):
        self.bot = bot

    evescout_commands = SlashCommandGroup(
        "evescout", "EVE-Scouts", guild_ids=[int(settings.DISCORD_GUILD_ID)]
    )

    @evescout_commands.command(
        name="initiate-pinger",
        description="Initiate the configuration for pings when a Thera/Turnur connection is reported",
    )
    @sender_has_perm("evescout.create_pinger")
    async def initiate_pinger(self, ctx: ApplicationContext):
        """Creates a new pinger and link it to the channel the command was called in"""
        channel_id = ctx.channel_id
        logger.info("Creating a pinger for channel id %s", channel_id)

        pinger = SignaturePinger.create(channel_id)

        site_url = get_site_url()
        pinger_url = f"{site_url}/admin/evescout/signaturepinger/{pinger.id}"

        return await ctx.respond(
            f"Successfully created a new pinger. You can edit it at {pinger_url}"
        )

    @evescout_commands.command(
        name="closest-signature",
        description="Finds the closest Thera/Turnur connection from the selected system",
    )
    @option(
        "system",
        parameter_name="system_name",
        description="System to search from",
        autocomplete=search_solar_system,
    )
    @option(
        "origin",
        parameter_name="signature_origin_str",
        required=False,
        description="Origin of the closest connection you want (can be empty)",
        autocomplete=possible_origins,
    )
    async def closest_signature(
        self, ctx: ApplicationContext, system_name: str, signature_origin_str: str
    ):
        logger.debug(
            "Received a closest_signature command with arguments %s, %s",
            system_name,
            signature_origin_str,
        )

        await ctx.defer()

        solar_system = find_solar_system_from_str(system_name)
        if not solar_system:
            logger.error("Couldn't find a solar system matching name %s", system_name)
            return await ctx.respond(
                f"Couldn't find a solar system matching name `{system_name}`"
            )

        if solar_system.is_w_space:
            logger.error("Won't search connections for wh system %s", solar_system.name)
            return await ctx.respond("Can't search for wh systems")
        elif solar_system.is_trig_space:
            logger.error(
                "Won't search connections for trig system %s", solar_system.name
            )
            return await ctx.respond("Can't search for triglavian systems")

        logger.info(f"Searching connections close to {solar_system.name}")

        e = default_embed(f"Connections close to {solar_system.name}")

        if signature_origin_str in (None, "THERA"):
            for signature in SignatureSystem.get_closest_signature(
                solar_system, SignatureSystem.SignatureOrigin.THERA
            ):
                e.add_field(
                    name=f"Thera connection in {signature.system.name}",
                    value=_signature_info(signature, solar_system),
                )

        if signature_origin_str in (None, "TURNUR"):
            for signature in SignatureSystem.get_closest_signature(
                solar_system, SignatureSystem.SignatureOrigin.TURNUR
            ):
                e.add_field(
                    name=f"Turnur connection in {signature.system.name}",
                    value=_signature_info(signature, solar_system),
                )

        await ctx.respond(embed=e)

    @evescout_commands.command(
        name="drifters", description="List drifter observatories arround a system"
    )
    @option(
        "system",
        parameter_name="system_name",
        description="System to search from",
        autocomplete=search_solar_system,
    )
    async def drifter_orbservatories(self, ctx: ApplicationContext, system_name: str):
        logger.info("Returning drifter wormholes around %s", system_name)
        await ctx.defer()

        solar_system = find_solar_system_from_str(system_name)

        routes = routes_jove_observatories(solar_system)

        e = default_embed(f"Jove observatories around {solar_system.name}")

        for system in routes:
            e.add_field(name=system["to"], value=f"{system['jumps']} jumps out")

        return await ctx.respond(embed=e)

    @evescout_commands.command(
        name="region-observatories",
        description="Returns the dotlan region URL with all observatory systems selected",
    )
    @option(
        "region-name",
        parameter_name="region_name",
        description="Name of the region to return",
        autocomplete=possible_regions,
    )
    async def region_observatories(self, ctx: ApplicationContext, region_name: str):
        logger.info("Trying to return the observatories in the region %s", region_name)

        try:
            embed_title = f"Jove observatories in {region_name}"
        except KeyError:
            embed_title = f"{region_name} is not a valid region name"

        e = default_embed(embed_title)
        e.add_field(name="dotlan", value=REGION_OBS[region_name])
        e.set_footer(text="The data isn't perfect. Please repport any mistake.")

        return await ctx.respond(embed=e)


def setup(bot):
    bot.add_cog(EveScout(bot))



# Generated by Django 4.2.15 on 2024-11-03 23:40

import django.core.validators
import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("eveuniverse", "0010_alter_eveindustryactivityduration_eve_type_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="General",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "permissions": (
                    ("create_pinger", "Can create a pinger in a discord channel"),
                ),
                "managed": False,
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="Singleton",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "last_check",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        help_text="Stores the last time we had new API data to work with",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="SignatureSystem",
            fields=[
                (
                    "id",
                    models.IntegerField(
                        help_text="Eve scout signature id",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "origin",
                    models.CharField(
                        choices=[("TH", "Thera"), ("TU", "Turnur")],
                        help_text="Where the signature originates from",
                        max_length=2,
                    ),
                ),
                (
                    "size",
                    models.CharField(
                        choices=[
                            ("C", "Capital"),
                            ("XL", "Extra-Large"),
                            ("L", "Large"),
                            ("M", "Medium"),
                            ("S", "Small"),
                        ],
                        help_text="Size of the wormhole",
                        max_length=2,
                        null=True,
                    ),
                ),
                (
                    "system",
                    models.ForeignKey(
                        help_text="System in which the signature appeared",
                        on_delete=django.db.models.deletion.CASCADE,
                        to="eveuniverse.evesolarsystem",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="SignaturePinger",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "discord_channel_id",
                    models.BigIntegerField(help_text="Discord channel id to ping"),
                ),
                (
                    "ping_for_thera",
                    models.BooleanField(
                        default=True,
                        help_text="If there should be a ping for Thera connections",
                    ),
                ),
                (
                    "ping_for_turnur",
                    models.BooleanField(
                        default=True,
                        help_text="If there should be a ping for Turnur connections",
                    ),
                ),
                (
                    "ping_type",
                    models.CharField(
                        choices=[
                            ("", "None"),
                            ("@here", "@here"),
                            ("@everyone", "@everyone"),
                        ],
                        default="",
                        help_text="Type of notification that should be sent out with the ping",
                        max_length=9,
                    ),
                ),
                (
                    "always_ping",
                    models.BooleanField(
                        default=False,
                        help_text="Will bypass the minimum distances and always ping when a new signature is detected.System selection still applies",
                    ),
                ),
                (
                    "min_ping_distance_ly",
                    models.FloatField(
                        default=0,
                        help_text="Ping if the signature spawns less than this amount of light years",
                        validators=[django.core.validators.MinValueValidator(0)],
                    ),
                ),
                (
                    "min_ping_distance_jump",
                    models.IntegerField(
                        default=0,
                        help_text="ping if the signature spawns less than this amount of jumps",
                        validators=[django.core.validators.MinValueValidator(0)],
                    ),
                ),
                (
                    "system",
                    models.ForeignKey(
                        help_text="System to be watched",
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="eveuniverse.evesolarsystem",
                    ),
                ),
            ],
        ),
    ]



"""Models."""

import sys
from typing import Iterable, Optional

from aadiscordbot.tasks import send_message
from solo.models import SingletonModel

from django.core.validators import MinValueValidator
from django.db import models
from django.utils import timezone
from eveuniverse.helpers import meters_to_ly
from eveuniverse.models import EveSolarSystem

from allianceauth.services.hooks import get_extension_logger

from evescout.utils import default_embed

logger = get_extension_logger(__name__)


class General(models.Model):
    """A meta model for app permissions."""

    class Meta:
        managed = False
        default_permissions = ()
        permissions = (("create_pinger", "Can create a pinger in a discord channel"),)


class Singleton(SingletonModel):
    """Singleton preserving information on the previous requests"""

    last_check = models.DateTimeField(
        default=timezone.now,
        help_text="Stores the last time we had new API data to work with",
    )

    @classmethod
    def get_last_check_time(cls):
        """Get the last time signatures were checked"""
        return cls.get_solo().last_check

    @classmethod
    def update_check_time(cls):
        """Update `last_check` to now after updating data correctly"""
        singleton = cls.get_solo()
        singleton.last_check = timezone.now()
        singleton.save()

    def __str__(self):
        return "Evescout Signleton"


class SignaturePinger(models.Model):
    """System to be watched and ping if a signature appear close to this system"""

    class PingType(models.TextChoices):
        """Ways to ping on discord"""

        NONE = " ", "None"
        HERE = "@here", "@here"
        EVERYONE = "@everyone", "@everyone"

    system = models.ForeignKey(
        EveSolarSystem,
        null=True,
        on_delete=models.CASCADE,
        help_text="System to be watched",
    )
    discord_channel_id = models.BigIntegerField(help_text="Discord channel id to ping")

    ping_for_thera = models.BooleanField(
        default=True, help_text="If there should be a ping for Thera connections"
    )
    ping_for_turnur = models.BooleanField(
        default=True, help_text="If there should be a ping for Turnur connections"
    )

    ping_type = models.CharField(
        max_length=9,
        choices=PingType.choices,
        default=PingType.NONE,
        help_text="Type of notification that should be sent out with the ping",
    )

    always_ping = models.BooleanField(
        default=False,
        help_text="Will bypass the minimum distances and always ping when a new signature is detected."
        "System selection still applies",
    )

    min_ping_distance_ly = models.FloatField(
        default=0,
        help_text="Ping if the signature spawns less than this amount of light years",
        validators=[
            MinValueValidator(0),
        ],
    )
    min_ping_distance_jump = models.IntegerField(
        default=0,
        help_text="ping if the signature spawns less than this amount of jumps",
        validators=[
            MinValueValidator(0),
        ],
    )

    @classmethod
    def create(
        cls, discord_channel_id: int, solar_system: Optional[EveSolarSystem] = None
    ) -> "SignaturePinger":
        """Instantiate new pinger"""
        return cls.objects.create(
            discord_channel_id=discord_channel_id, system=solar_system
        )

    def ping_new_sig(self, signature: "SignatureSystem"):
        """Sends a message on discord to warn for a new signature"""
        gate_jumps, light_years = signature.distance_from_pinger(self)
        e = default_embed(
            f"New signature from {signature.origin_system_name()} detected in {signature.system.name}"
        )
        e.add_field(
            name=f"Distance from {self.system.name}",
            value=f"{gate_jumps} jumps / {round(light_years, 2)} ly.",
        )
        e.add_field(
            name="Wormhole size",
            value=SignatureSystem.WormholeSize.from_value_to_label(signature.size),
        )
        send_message(
            channel_id=self.discord_channel_id, embed=e, message=self.ping_type
        )

    def __str__(self):
        if system := self.system:
            return f"Pinger {system.name}"
        return f"Unlinked pinger {self.id}"


class SignatureSystem(models.Model):
    """Represents a system in which a sig was detected"""

    class SignatureOrigin(models.TextChoices):
        """Possible origins of a wormhole"""

        THERA = "TH", "Thera"
        TURNUR = "TU", "Turnur"

    class WormholeSize(models.TextChoices):
        """Possible size of the wormhole"""

        C = "C", "Capital"
        XL = "XL", "Extra-Large"
        L = "L", "Large"
        M = "M", "Medium"
        S = "S", "Small"

        @classmethod
        def from_value_to_label(cls, value: str) -> str:
            """Return the label from one of the possible values"""
            return getattr(cls, value).label

    id = models.IntegerField(primary_key=True, help_text="Eve scout signature id")

    system = models.ForeignKey(
        EveSolarSystem,
        on_delete=models.CASCADE,
        help_text="System in which the signature appeared",
    )
    origin = models.CharField(
        max_length=2,
        choices=SignatureOrigin.choices,
        help_text="Where the signature originates from",
    )
    size = models.CharField(
        max_length=2,
        null=True,
        choices=WormholeSize.choices,
        help_text="Size of the wormhole",
    )

    @classmethod
    def create(
        cls,
        signature_id: int,
        system: EveSolarSystem,
        origin: SignatureOrigin,
        size: Optional[WormholeSize] = None,
    ) -> "SignatureSystem":
        """Creates a new signature"""
        logger.debug(
            "Creating a signature of size %s in %s from %s with id %s",
            size,
            system,
            origin,
            signature_id,
        )
        return cls.objects.create(
            id=signature_id, system=system, origin=origin, size=size
        )

    @classmethod
    def get_signatures_from(cls, system: SignatureOrigin) -> list["SignatureSystem"]:
        """Returns all signatures linked to a system"""
        return cls.objects.filter(origin=system)

    @classmethod
    def get_signature_ids_set(cls) -> set[int]:
        """Return a set with the signature id of all signatures in the database"""
        id_list = cls.objects.values_list("id", flat=True)
        return set(id_list)

    @classmethod
    def delete_disappeared_signatures(cls, existing_signatures_ids: Iterable[int]):
        """Gets a list of signature ids. Will delete all Signatures that are not part of this list"""

        disappeared_signatures = cls.objects.exclude(id__in=existing_signatures_ids)
        disappeared_signatures.delete()

        logger.debug("Deleted % signature(s)", disappeared_signatures.count())

    @classmethod
    def get_closest_signature(
        cls, solar_system: EveSolarSystem, origin: SignatureOrigin
    ) -> list["SignatureSystem"]:
        """
        Will return the closes signature linked to `origin` from `solar_system`
        Return is made as a list to be able to return several systems if they are at similar ranges
        """
        best_range = sys.maxsize
        best_systems = []
        for signature_system in cls.get_signatures_from(origin):
            distance_gates, _ = signature_system.distance_from_solar_system(
                solar_system
            )
            if distance_gates < best_range:
                best_range = distance_gates
                best_systems = [signature_system]
            elif distance_gates == best_range:
                best_systems.append(signature_system)

        return best_systems

    def origin_system_name(self) -> str:
        """Returns the human-readable origin system"""
        match self.origin:
            case self.SignatureOrigin.THERA:
                return "Thera"
            case self.SignatureOrigin.TURNUR:
                return "Turnur"

    def distance_from_solar_system(self, solar_system: EveSolarSystem) -> (int, float):
        """
        Return the distance from the signature and a solar system in gate jumps and light years
        """
        if route := solar_system.route_to(self.system):
            gate_jumps = len(route) - 1  # -1 to avoid counting the initial system
            light_years = meters_to_ly(solar_system.distance_to(self.system))
            return gate_jumps, light_years

        return sys.maxsize, float("inf")

    def distance_from_pinger(self, pinger: SignaturePinger) -> (int, float):
        """
        Return the distance from the pinger system in gate jumps and light years
        """
        if pinger_system := pinger.system:
            return self.distance_from_solar_system(pinger_system)
        return sys.maxsize, float("inf")

    def is_pinger_in_range(self, pinger: SignaturePinger) -> bool:
        """True if the pinger is in rage for gates or jump range"""
        if self.system.is_trig_space or self.system.is_w_space:
            return False

        if pinger.always_ping:
            return True

        gate_jumps, light_years = self.distance_from_pinger(pinger)
        return (
            gate_jumps <= pinger.min_ping_distance_jump
            or light_years <= pinger.min_ping_distance_ly
        )

    def pingers_in_range(self) -> list[SignaturePinger]:
        """Returns all pingers with their get/distance parameters near the signature"""
        return [
            pinger
            for pinger in SignaturePinger.objects.all()
            if self.is_pinger_in_range(pinger)
        ]

    def get_size_str(self) -> str:
        """Get a readable size"""
        return SignatureSystem.WormholeSize.from_value_to_label(self.size)

    def __str__(self):
        return f"Signature {self.origin} > {self.system}"



import responses

from django.test import TestCase
from eveuniverse.models import EveSolarSystem

from evescout.models import SignatureSystem, Singleton
from evescout.tasks import create_new_signature, update_all_signatures
from evescout.tests.testdata.load_eveuniverse import load_eveuniverse


class TestTasks(TestCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_eveuniverse()

    def test_create_new_signature(self):
        nagamanen = EveSolarSystem.objects.get(id=30002807)

        # Edited signature to match loaded information
        signature_info = {
            "id": "24352",
            "created_at": "2024-11-01T07:21:44.000Z",
            "created_by_id": 2121887172,
            "created_by_name": "Kiera Endymion",
            "updated_at": "2024-11-01T07:26:04.000Z",
            "updated_by_id": 2121887172,
            "updated_by_name": "Kiera Endymion",
            "completed_at": "2024-11-01T07:26:04.000Z",
            "completed_by_id": 2121887172,
            "completed_by_name": "Kiera Endymion",
            "completed": True,
            "wh_exits_outward": False,
            "wh_type": "F135",
            "max_ship_size": "large",
            "expires_at": "2024-11-02T00:21:44.000Z",
            "remaining_hours": 1,
            "signature_type": "wormhole",
            "out_system_id": 31000005,
            "out_system_name": "Thera",
            "out_signature": "GBO-896",
            "in_system_id": 30002807,
            "in_system_class": "ls",
            "in_system_name": "Nagamanen",
            "in_region_id": 10000033,
            "in_region_name": "The Citadel",
            "in_signature": "NNA-392",
        }

        create_new_signature(signature_info)

        signature = SignatureSystem.objects.get(id=24352)

        self.assertEqual(signature.system, nagamanen)
        self.assertEqual(signature.origin, SignatureSystem.SignatureOrigin.THERA)
        self.assertEqual(signature.size, SignatureSystem.WormholeSize.L)

    @responses.activate
    def test_duplicate_signature(self):
        """
        Test that there's no crash or duplicate when an already registered signature is entered
        """
        nagamanen = EveSolarSystem.objects.get(id=30002807)

        SignatureSystem.create(24352, nagamanen, SignatureSystem.SignatureOrigin.THERA)

        # Edited signature to match loaded information
        response_content = [
            {
                "id": "24352",
                "created_at": "2024-11-01T07:21:44.000Z",
                "created_by_id": 2121887172,
                "created_by_name": "Kiera Endymion",
                "updated_at": "2024-11-01T07:26:04.000Z",
                "updated_by_id": 2121887172,
                "updated_by_name": "Kiera Endymion",
                "completed_at": "2024-11-01T07:26:04.000Z",
                "completed_by_id": 2121887172,
                "completed_by_name": "Kiera Endymion",
                "completed": True,
                "wh_exits_outward": False,
                "wh_type": "F135",
                "max_ship_size": "large",
                "expires_at": "2024-11-02T00:21:44.000Z",
                "remaining_hours": 1,
                "signature_type": "wormhole",
                "out_system_id": 31000005,
                "out_system_name": "Thera",
                "out_signature": "GBO-896",
                "in_system_id": 30002807,
                "in_system_class": "ls",
                "in_system_name": "Nagamanen",
                "in_region_id": 10000033,
                "in_region_name": "The Citadel",
                "in_signature": "NNA-392",
            }
        ]

        responses.get(
            "https://api.eve-scout.com/v2/public/signatures", json=response_content
        )

        update_all_signatures()

        self.assertEqual(1, SignatureSystem.objects.count())

    @responses.activate
    def test_delete_signature(self):
        # Imaginary signature with non existing id
        SignatureSystem.create(
            00, EveSolarSystem.objects.all()[0], SignatureSystem.SignatureOrigin.THERA
        )

        response_content = []

        responses.get(
            "https://api.eve-scout.com/v2/public/signatures", json=response_content
        )

        update_all_signatures()

        self.assertEqual(0, SignatureSystem.objects.count())

    @responses.activate
    def test_no_update_if_after_check(self):
        """
        Checks that the database isn't altered if the last_check flag is bigger than the header value
        """

        # Edited signature to match loaded information
        response_content = [
            {
                "id": "24352",
                "created_at": "2024-11-01T07:21:44.000Z",
                "created_by_id": 2121887172,
                "created_by_name": "Kiera Endymion",
                "updated_at": "2024-11-01T07:26:04.000Z",
                "updated_by_id": 2121887172,
                "updated_by_name": "Kiera Endymion",
                "completed_at": "2024-11-01T07:26:04.000Z",
                "completed_by_id": 2121887172,
                "completed_by_name": "Kiera Endymion",
                "completed": True,
                "wh_exits_outward": False,
                "wh_type": "F135",
                "max_ship_size": "large",
                "expires_at": "2024-11-02T00:21:44.000Z",
                "remaining_hours": 1,
                "signature_type": "wormhole",
                "out_system_id": 31000005,
                "out_system_name": "Thera",
                "out_signature": "GBO-896",
                "in_system_id": 30002807,
                "in_system_class": "ls",
                "in_system_name": "Nagamanen",
                "in_region_id": 10000033,
                "in_region_name": "The Citadel",
                "in_signature": "NNA-392",
            }
        ]

        response_header = {
            "x-last-signaleer-hub-interaction": "2004-11-08T15:13:09.052Z",  # value well in the past
        }

        responses.get(
            "https://api.eve-scout.com/v2/public/signatures",
            json=response_content,
            headers=response_header,
        )

        Singleton.update_check_time()

        update_all_signatures()

        self.assertEqual(0, SignatureSystem.objects.count())

        update_all_signatures(ignore_last_check=True)

        self.assertEqual(1, SignatureSystem.objects.count())
